!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!   ##########################################################################
    SUBROUTINE URBAN_LW_COEF(B, T, PLW_RAD, PEMIS_G, PTS_SR, PTS_G,               &
                             PEMIS_HV, PTS_HV,                                    &  
                             PLW_WA_TO_WB, PLW_WA_TO_R, PLW_WB_TO_R,              &
                             PLW_WA_TO_NR,PLW_WB_TO_NR,                           &
                             PLW_WA_TO_G, PLW_WB_TO_G,                            &
                             PLW_WA_TO_WIN, PLW_WB_TO_WIN,                        &
                             PLW_WA_TO_HV, PLW_WB_TO_HV,                          &
                             PLW_WA_TO_S, PLW_WB_TO_S,                            &
                             PLW_R_TO_WA, PLW_R_TO_WB, PLW_R_TO_WIN, PLW_R_TO_HV, &
                             PLW_R_TO_S,                                          &
                             PLW_G_TO_WA, PLW_G_TO_WB, PLW_G_TO_WIN, PLW_G_TO_HV, &
                             PLW_G_TO_S,                                          &
                             PLW_S_TO_WA, PLW_S_TO_WB, PLW_S_TO_R, PLW_S_TO_NR,   &
                             PLW_S_TO_G, PLW_S_TO_WIN, PLW_S_TO_HV,               &
                             PLW_WIN_TO_WA, PLW_WIN_TO_WB,                        &
                             PLW_WIN_TO_R, PLW_WIN_TO_NR,                         &
                             PLW_WIN_TO_G, PLW_WIN_TO_HV, PLW_WIN_TO_S,           &
                             PLW_NR_TO_WA, PLW_NR_TO_WB, PLW_NR_TO_WIN,           &
                             PLW_NR_TO_HV, PLW_NR_TO_S,                           &
                             PLW_HV_TO_WA, PLW_HV_TO_WB, PLW_HV_TO_R,             &
                             PLW_HV_TO_NR,PLW_HV_TO_G, PLW_HV_TO_WIN, PLW_HV_TO_S )
!   ##########################################################################
!
!!****  *URBAN_LW_COEF*  
!!
!!    PURPOSE
!!    -------
!
!     Computes the coefficients before each of the temperatures in the
!     radiative budgets
!         
!     
!!**  METHOD
!     ------
!
!
!
!
!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    MODD_CST
!!
!!      
!!    REFERENCE
!!    ---------
!!
!!      
!!    AUTHOR
!!    ------
!!
!!      V. Masson           * Meteo-France *
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    08/09/98 
!-------------------------------------------------------------------------------
!
!*       0.     DECLARATIONS
!               ------------
!
USE MODD_BEM_n, ONLY : BEM_t
USE MODD_TEB_n, ONLY : TEB_t
!
USE MODD_TEB_PAR, ONLY : XEMIS_WIN_CST
USE MODD_CSTS,ONLY : XSTEFAN
USE MODD_SURF_PAR,ONLY : XUNDEF
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*      0.1    declarations of arguments
!
TYPE(BEM_t), INTENT(INOUT) :: B
TYPE(TEB_t), INTENT(INOUT) :: T
!
REAL, DIMENSION(:), INTENT(IN)  :: PLW_RAD  ! incoming LW radiation
REAL, DIMENSION(:), INTENT(IN)  :: PEMIS_G  ! GARDEN area emissivity
REAL, DIMENSION(:), INTENT(IN)  :: PEMIS_HV ! high vegetation emissivity
!
REAL, DIMENSION(:), INTENT(IN)  :: PTS_G    ! garden surface temperature
REAL, DIMENSION(:), INTENT(IN)  :: PTS_SR   ! snow surface temperature
REAL, DIMENSION(:), INTENT(IN)  :: PTS_HV   ! high vegetation foliage temperature
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_WB! L.W. interactions wall->opposite wall
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_R  ! L.W. interactions wall->road for road balance 
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_R  ! L.W. interactions wall->road for road balance 
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_NR ! L.W. interactions wall->snow for snow balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_NR ! L.W. interactions wall->snow for snow balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_G  ! L.W. interactions wall->GARDEN areas for garden balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_G  ! L.W. interactions wall->GARDEN areas for garden balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_WIN! L.W. interactions wall->win for window balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_WIN! L.W. interactions wall->win for window balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_HV  ! L.W. interactions wall->high vegetation areas for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_HV  ! L.W. interactions wall->high vegetation areas for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WA_TO_S   ! L.W. interactions wall->sky for sky balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WB_TO_S   ! L.W. interactions wall->sky for sky balance
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_R_TO_WA ! L.W. interactions road->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_R_TO_WB ! L.W. interactions road->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_R_TO_WIN  ! L.W. interactions road->win for win balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_R_TO_HV   ! L.W. interactions road->high vegetation for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_R_TO_S    ! L.W. interactions road->sky for sky balance
!
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_G_TO_WA ! L.W. interactions GARDEN areas->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_G_TO_WB ! L.W. interactions GARDEN areas->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_G_TO_WIN  ! L.W. interactions GARDEN areas->road for window balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_G_TO_HV   ! L.W. interactions GARDEN areas->high vegetation for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_G_TO_S    ! L.W. interactions GARDEN areas->sky for sky balance
!
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_WA ! L.W. interactions sky->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_WB ! L.W. interactions sky->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_R  ! L.W. interactions sky->road for raod balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_NR ! L.W. interactions sky->snow for snow balance 
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_G  ! L.W. interactions sky->GARDEN areas for garden balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_WIN ! L.W. interactions sky->win for window balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_S_TO_HV  ! L.W. interactions sky->high vegetation areas for high veg balance
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_WA ! L.W. interactions win->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_WB ! L.W. interactions win->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_R  ! L.W. interactions win->road for road balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_NR ! L.W. interactions win->GARDEN areas for snow balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_G  ! L.W. interactions win->GARDEN areas for garden balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_HV ! L.W. interactions win->high vegetation areas for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_WIN_TO_S  ! L.W. interactions win->sky for sky balance
!
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_NR_TO_WA! L.W. interactions snow(road)->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_NR_TO_WB! L.W. interactions snow(road)->wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_NR_TO_WIN ! L.W. interactions snow(road)->WIN areas for window balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_NR_TO_HV  ! L.W. interactions snow(road)->high vegetation for high veg balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_NR_TO_S   ! L.W. interactions snow(road)->sky for sky balance
!
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_WA  ! L.W. interaction high veg -> wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_WB  ! L.W. interaction high veg -> wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_WIN ! L.W. interaction high veg -> wall for wall balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_G   ! L.W. interaction high veg -> garden for garden balance
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_R   ! L.W. interaction high veg -> road for road balance 
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_NR  ! L.W. interaction high veg -> snow for road balance 
REAL, DIMENSION(:), INTENT(OUT) :: PLW_HV_TO_S   ! L.W. interaction high veg -> sky for sky balance 
!
!
!*      0.2    declarations of local variables
!
REAL, DIMENSION(SIZE(T%XBLD))  :: ZT_S        ! sky temperature
REAL, DIMENSION(SIZE(T%XBLD))  :: ZEMIS_WIN   ! window emissivity
REAL, DIMENSION(SIZE(T%XBLD))  :: ZROAD       ! road   fraction relative to canyon fraction
REAL, DIMENSION(SIZE(T%XBLD))  :: ZGARDEN     ! garden fraction relative to canyon fraction
!
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_WIN
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_R
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_G
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_NR
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_W_HV
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_WIN_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_WIN_R
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_WIN_G
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_WIN_NR
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_WIN_HV
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_R_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_R_WIN
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_R_HV
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_G_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_G_WIN
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_G_HV
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_HV_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_HV_WIN
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_HV_R
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_HV_G
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_W
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_R
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_G
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_NR
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_WIN
REAL, DIMENSION(SIZE(T%XBLD))  :: ZF_S_HV
!
REAL :: ZE1, ZE2, ZF, ZT1, ZT2, ZLW, ZTAU
ZLW(ZE1,ZE2,ZF,ZT1,ZT2,ZTAU) = 4.*XSTEFAN*ZE1*ZE2*ZF*ZTAU*((ZT1+ZT2)/2.)**3
!
INTEGER :: JJ
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('URBAN_LW_COEF',0,ZHOOK_HANDLE)
!
ZT_S(:) = (PLW_RAD(:)/XSTEFAN)**0.25
!
ZEMIS_WIN(:) = XEMIS_WIN_CST

DO JJ=1,SIZE(T%XROAD)
  !
  ZF_W_R(JJ)     = T%XSVF_WR(JJ) * (T%XROAD(JJ)   / (1.-T%XBLD(JJ) )) ! without considering snow cover at this stage (done in wall_layer_e_budget) 
  ZF_W_G(JJ)     = T%XSVF_WR(JJ) * (T%XGARDEN(JJ) / (1.-T%XBLD(JJ) ))
  ZF_W_NR(JJ)    = ZF_W_R (JJ)                                  ! without considering snow cover at this stage (done in wall_layer_e_budget)   
  ZF_W_W(JJ)     = T%XSVF_WW(JJ) * (1.-B%XGR(JJ))
  ZF_W_WIN(JJ)   = T%XSVF_WW(JJ) *     B%XGR(JJ)
  ! 
  ZF_WIN_R(JJ)   = ZF_W_R (JJ)
  ZF_WIN_G(JJ)   = ZF_W_G (JJ)
  ZF_WIN_NR(JJ)  = ZF_W_NR(JJ)
  ZF_WIN_W(JJ)   = ZF_W_W (JJ)
  !
  ZF_R_W(JJ)     = T%XSVF_RW(JJ) * (1.-B%XGR(JJ)) * 0.5 ! one wall can emit towards the road - idem for garden
  ZF_R_WIN(JJ)   = T%XSVF_RW(JJ) *     B%XGR(JJ)
  !
  ZF_G_W(JJ)     = ZF_R_W(JJ) 
  ZF_G_WIN(JJ)   = ZF_R_WIN(JJ)
  !
  ZF_S_R(JJ)     = T%XSVF_RS(JJ) * (T%XROAD(JJ)   / (1.-T%XBLD(JJ) )) ! without considering snow cover at this stage (done in wall_layer_e_budget) 
  ZF_S_G(JJ)     = T%XSVF_RS(JJ) * (T%XGARDEN(JJ) / (1.-T%XBLD(JJ) ))
  ZF_S_NR(JJ)    = ZF_S_R(JJ)                                  ! without considering snow cover at this stage (done in wall_layer_e_budget)   
  ZF_S_W  (JJ)   = T%XSVF_SW(JJ) * (1.-B%XGR(JJ)) * 0.5
  ZF_S_WIN(JJ)   = T%XSVF_SW(JJ) *     B%XGR(JJ)
!
  ZF_W_HV(JJ)    = T%XSVF_WT(JJ) * 0.5 * T%XURBTREE(JJ) ! we corrected the hveg contribution by 0.5 because svf_wt is defined on 360° or 2pi
  ZF_WIN_HV(JJ)  = ZF_W_HV(JJ)
  ZF_R_HV(JJ)    = T%XSVF_RT(JJ) * T%XURBTREE(JJ)
  ZF_G_HV(JJ)    = ZF_R_HV(JJ)
  ZF_S_HV(JJ)    = T%XSVF_ST(JJ) * T%XURBTREE(JJ)
!
  IF (T%XURBTREE(JJ) .GT. 0.) THEN ! we apply the same geometrical factors as in the solar radiative scheme
!
    ZROAD(JJ)   = T%XROAD(JJ)   / (1.-T%XBLD(JJ))
    ZGARDEN(JJ) = T%XGARDEN(JJ) / (1.-T%XBLD(JJ))
!
!   sum of what has been intercepted (not transmitted) by high veg on the ray way
    ZF_HV_W(JJ)   =  ( T%XSVF_SW(JJ) * (1.- T%XTAU_SW(JJ) )   &
                      +T%XSVF_RW(JJ) * (1.- T%XTAU_WR(JJ) )   &
                      +T%XSVF_WW(JJ) * (1.- T%XTAU_WW(JJ) ) ) &
                    * (         1. /  T%XURBTREE(JJ)    )   &
                    * (1.-B%XGR(JJ)) * 0.5
!
    ZF_HV_WIN(JJ) =  ( T%XSVF_SW(JJ) * (1.- T%XTAU_SW(JJ) )   &
                      +T%XSVF_RW(JJ) * (1.- T%XTAU_WR(JJ) )   &
                      +T%XSVF_WW(JJ) * (1.- T%XTAU_WW(JJ) ) ) &
                    * (         1. /  T%XURBTREE(JJ)    )   &
                    * B%XGR(JJ)
!
    ZF_HV_R(JJ)   =  ( T%XSVF_RS(JJ) * (1.- T%XTAU_SR(JJ) )   &
                      +T%XSVF_WR(JJ) * (1.- T%XTAU_WR(JJ) ) ) & 
                    *( ZROAD(JJ)   /  T%XURBTREE(JJ)    )
!
    ZF_HV_G(JJ)   =  ( T%XSVF_RS(JJ) * (1.- T%XTAU_SR(JJ) )   &
                      +T%XSVF_WR(JJ) * (1.- T%XTAU_WR(JJ) ) ) & 
                    *( ZGARDEN(JJ) /  T%XURBTREE(JJ)    )
!
  ELSE 
    ZF_HV_W(JJ)   = XUNDEF 
    ZF_HV_WIN(JJ) = XUNDEF 
    ZF_HV_R(JJ)   = XUNDEF 
    ZF_HV_G(JJ)   = XUNDEF 
  ENDIF
!
! considering middle of foliage layer in tau terms
  !
  PLW_WA_TO_WB(JJ)  = ZLW(T%XEMIS_WALL(JJ),T%XEMIS_WALL(JJ),ZF_W_W(JJ),  T%XT_WALL_A(JJ,1),T%XT_WALL_B(JJ,1),T%XTAU_WW(JJ))
  PLW_WA_TO_R(JJ)   = ZLW(T%XEMIS_WALL(JJ),T%XEMIS_ROAD(JJ),ZF_R_W(JJ),  T%XT_WALL_A(JJ,1),T%XT_ROAD(JJ,1),  T%XTAU_WR(JJ))
  PLW_WA_TO_WIN(JJ) = ZLW(T%XEMIS_WALL(JJ),ZEMIS_WIN(JJ),   ZF_WIN_W(JJ),T%XT_WALL_A(JJ,1),B%XT_WIN1(JJ),    T%XTAU_WW(JJ))
  PLW_WA_TO_S(JJ)   = ZLW(T%XEMIS_WALL(JJ),1.,              ZF_S_W(JJ),  T%XT_WALL_A(JJ,1),ZT_S(JJ),         T%XTAU_SW(JJ))
  !
  PLW_WB_TO_R(JJ)   = ZLW(T%XEMIS_WALL(JJ),T%XEMIS_ROAD(JJ),ZF_R_W(JJ),  T%XT_WALL_B(JJ,1),T%XT_ROAD(JJ,1),T%XTAU_WR(JJ))
  PLW_WB_TO_WIN(JJ) = ZLW(T%XEMIS_WALL(JJ),ZEMIS_WIN(JJ),   ZF_WIN_W(JJ),T%XT_WALL_B(JJ,1),B%XT_WIN1(JJ),  T%XTAU_WW(JJ))
  PLW_WB_TO_S(JJ)   = ZLW(T%XEMIS_WALL(JJ),1.,              ZF_S_W(JJ),  T%XT_WALL_B(JJ,1),ZT_S(JJ),       T%XTAU_SW(JJ))
  !
  PLW_R_TO_WA(JJ) = ZLW(T%XEMIS_ROAD(JJ),T%XEMIS_WALL(JJ),ZF_W_R(JJ),  T%XT_ROAD(JJ,1),T%XT_WALL_A(JJ,1),T%XTAU_WR(JJ))
  PLW_R_TO_WB(JJ) = ZLW(T%XEMIS_ROAD(JJ),T%XEMIS_WALL(JJ),ZF_W_R(JJ),  T%XT_ROAD(JJ,1),T%XT_WALL_B(JJ,1),T%XTAU_WR(JJ))
  PLW_R_TO_WIN(JJ)= ZLW(T%XEMIS_ROAD(JJ),ZEMIS_WIN(JJ),   ZF_WIN_R(JJ),T%XT_ROAD(JJ,1),B%XT_WIN1(JJ),    T%XTAU_WR(JJ))
  PLW_R_TO_S(JJ)  = ZLW(T%XEMIS_ROAD(JJ),1.,              ZF_S_R(JJ),  T%XT_ROAD(JJ,1),ZT_S(JJ),         T%XTAU_SR(JJ))

  !  
  PLW_WIN_TO_WA(JJ) = ZLW(ZEMIS_WIN(JJ),T%XEMIS_WALL(JJ), ZF_W_WIN(JJ), B%XT_WIN1(JJ),T%XT_WALL_A(JJ,1),T%XTAU_WW(JJ))
  PLW_WIN_TO_WB(JJ) = ZLW(ZEMIS_WIN(JJ),T%XEMIS_WALL(JJ), ZF_W_WIN(JJ), B%XT_WIN1(JJ),T%XT_WALL_B(JJ,1),T%XTAU_WW(JJ))
  PLW_WIN_TO_R(JJ)  = ZLW(ZEMIS_WIN(JJ),T%XEMIS_ROAD(JJ), ZF_R_WIN(JJ), B%XT_WIN1(JJ),T%XT_ROAD(JJ,1)  ,T%XTAU_WR(JJ))
  PLW_WIN_TO_S(JJ)  = ZLW(ZEMIS_WIN(JJ),1.,               ZF_S_WIN(JJ), B%XT_WIN1(JJ),ZT_S(JJ),         T%XTAU_WR(JJ))
  !
  PLW_S_TO_WA(JJ)  = ZLW(1.,T%XEMIS_WALL(JJ),  T%XSVF_WS(JJ),ZT_S(JJ),T%XT_WALL_A(JJ,1),T%XTAU_SW(JJ))
  PLW_S_TO_WB(JJ)  = ZLW(1.,T%XEMIS_WALL(JJ),  T%XSVF_WS(JJ),ZT_S(JJ),T%XT_WALL_B(JJ,1),T%XTAU_SW(JJ))
  PLW_S_TO_R(JJ)   = ZLW(1.,T%XEMIS_ROAD(JJ),  T%XSVF_RS(JJ),ZT_S(JJ),T%XT_ROAD(JJ,1),  T%XTAU_SR(JJ))  
  PLW_S_TO_WIN(JJ) = ZLW(1.,ZEMIS_WIN(JJ),     T%XSVF_WS(JJ),ZT_S(JJ),B%XT_WIN1(JJ),    T%XTAU_SW(JJ))
  !
  IF (SIZE(PTS_G)>0) THEN
    PLW_WA_TO_G(JJ)   = ZLW(T%XEMIS_WALL(JJ),PEMIS_G(JJ),     ZF_G_W(JJ),  T%XT_WALL_A(JJ,1),PTS_G(JJ),T%XTAU_WR(JJ))
    PLW_WB_TO_G(JJ)   = ZLW(T%XEMIS_WALL(JJ),PEMIS_G(JJ),     ZF_G_W(JJ),  T%XT_WALL_B(JJ,1),PTS_G(JJ),T%XTAU_WR(JJ))
    !
    PLW_G_TO_WA(JJ) = ZLW(PEMIS_G(JJ),  T%XEMIS_WALL(JJ),ZF_W_G(JJ),   PTS_G(JJ),    T%XT_WALL_A(JJ,1),T%XTAU_WR(JJ))
    PLW_G_TO_WB(JJ) = ZLW(PEMIS_G(JJ),  T%XEMIS_WALL(JJ),ZF_W_G(JJ),   PTS_G(JJ),    T%XT_WALL_B(JJ,1),T%XTAU_WR(JJ))
    PLW_G_TO_WIN(JJ)= ZLW(PEMIS_G(JJ),  ZEMIS_WIN(JJ),   ZF_WIN_G(JJ), PTS_G(JJ),    B%XT_WIN1(JJ),    T%XTAU_WR(JJ))
    PLW_G_TO_S(JJ)  = ZLW(PEMIS_G(JJ),  1.,              ZF_S_G(JJ),   PTS_G(JJ),    ZT_S(JJ),         T%XTAU_SR(JJ))
    PLW_WIN_TO_G(JJ)= ZLW(ZEMIS_WIN(JJ),PEMIS_G(JJ),     ZF_G_WIN(JJ), B%XT_WIN1(JJ),PTS_G(JJ),        T%XTAU_WR(JJ))
    PLW_S_TO_G(JJ)  = ZLW(1.,           PEMIS_G(JJ),     T%XSVF_RS(JJ),ZT_S(JJ),     PTS_G(JJ),        T%XTAU_SR(JJ))
  ELSE
    PLW_WA_TO_G(JJ) = 0.
    PLW_WB_TO_G(JJ) = 0.
    !
    PLW_G_TO_WA(JJ) = 0.
    PLW_G_TO_WB(JJ) = 0.
    PLW_G_TO_WIN(JJ)= 0.
    PLW_G_TO_S(JJ)  = 0.
    PLW_WIN_TO_G(JJ)= 0.
    PLW_S_TO_G(JJ)  = 0.
  END IF

  IF (T%XURBTREE(JJ) .GT. 0.) THEN
  PLW_WA_TO_HV(JJ)  = ZLW(T%XEMIS_WALL(JJ),PEMIS_HV(JJ),    ZF_HV_W(JJ),  T%XT_WALL_A(JJ,1),PTS_HV(JJ),       1.)
  PLW_WB_TO_HV(JJ)  = ZLW(T%XEMIS_WALL(JJ),PEMIS_HV(JJ),    ZF_HV_W(JJ),  T%XT_WALL_B(JJ,1),PTS_HV(JJ),       1.)
  PLW_R_TO_HV(JJ)   = ZLW(T%XEMIS_ROAD(JJ),PEMIS_HV(JJ),    ZF_HV_R(JJ),  T%XT_ROAD(JJ,1),  PTS_HV(JJ),       1.)
  PLW_G_TO_HV(JJ)   = ZLW(PEMIS_G(JJ),     PEMIS_HV(JJ),    ZF_HV_G(JJ),  PTS_G(JJ),        PTS_HV(JJ),       1.)
  PLW_WIN_TO_HV(JJ) = ZLW(ZEMIS_WIN(JJ),   PEMIS_HV(JJ),    ZF_HV_WIN(JJ),B%XT_WIN1(JJ),    PTS_HV(JJ),       1.)
  PLW_S_TO_HV(JJ)   = ZLW(1.,              PEMIS_HV(JJ),    T%XSVF_TS(JJ),ZT_S(JJ),         PTS_HV(JJ),       1.)
  PLW_HV_TO_WA(JJ)  = ZLW(PEMIS_HV(JJ),    T%XEMIS_WALL(JJ),ZF_W_HV(JJ),  PTS_HV(JJ),       T%XT_WALL_A(JJ,1),1.)
  PLW_HV_TO_WB(JJ)  = ZLW(PEMIS_HV(JJ),    T%XEMIS_WALL(JJ),ZF_W_HV(JJ),  PTS_HV(JJ),       T%XT_WALL_B(JJ,1),1.) 
  PLW_HV_TO_R(JJ)   = ZLW(PEMIS_HV(JJ),    T%XEMIS_ROAD(JJ),ZF_R_HV(JJ),  PTS_HV(JJ),       T%XT_ROAD(JJ,1),  1.)
  PLW_HV_TO_G(JJ)   = ZLW(PEMIS_HV(JJ),    PEMIS_G(JJ),     ZF_G_HV(JJ),  PTS_HV(JJ),       PTS_G(JJ),        1.)
  PLW_HV_TO_WIN(JJ) = ZLW(PEMIS_HV(JJ),    ZEMIS_WIN(JJ),   ZF_WIN_HV(JJ),PTS_HV(JJ),       B%XT_WIN1(JJ),    1.)
  PLW_HV_TO_S(JJ) =   ZLW(PEMIS_HV(JJ),    1.,              ZF_S_HV(JJ),  PTS_HV(JJ),       ZT_S(JJ),         1.)
  ELSE
  PLW_WA_TO_HV(JJ)  = 0.
  PLW_WB_TO_HV(JJ)  = 0.
  PLW_R_TO_HV(JJ)   = 0.
  PLW_G_TO_HV(JJ)   = 0.
  PLW_WIN_TO_HV(JJ) = 0.
  PLW_S_TO_HV(JJ)   = 0.
  PLW_HV_TO_WA(JJ)  = 0.
  PLW_HV_TO_WB(JJ)  = 0.
  PLW_HV_TO_R(JJ)   = 0.
  PLW_HV_TO_G(JJ)   = 0.
  PLW_HV_TO_WIN(JJ) = 0.
  PLW_HV_TO_S(JJ)   = 0.
  ENDIF
  !
  IF (PTS_SR(JJ) .EQ. XUNDEF) THEN
    PLW_WA_TO_NR (JJ) = 0.
    PLW_WB_TO_NR (JJ) = 0.
    PLW_HV_TO_NR(JJ)  = 0.
    PLW_WIN_TO_NR(JJ) = 0.
    PLW_S_TO_NR  (JJ) = 0.
!
    PLW_NR_TO_WA (JJ) = 0.
    PLW_NR_TO_WB (JJ) = 0.
    PLW_NR_TO_HV(JJ)  = 0.
    PLW_NR_TO_WIN(JJ) = 0.
    PLW_NR_TO_S(JJ)   = 0.
!
  ELSE
!
    PLW_WA_TO_NR(JJ) = ZLW(T%XEMIS_WALL(JJ),  T%TSNOW_ROAD%EMIS(JJ),ZF_R_W(JJ),  T%XT_WALL_A(JJ,1),PTS_SR(JJ),T%XTAU_WR(JJ))
    PLW_WB_TO_NR(JJ) = ZLW(T%XEMIS_WALL(JJ),  T%TSNOW_ROAD%EMIS(JJ),ZF_R_W(JJ),  T%XT_WALL_B(JJ,1),PTS_SR(JJ),T%XTAU_WR(JJ))
    PLW_WIN_TO_NR(JJ)= ZLW(ZEMIS_WIN(JJ),         T%TSNOW_ROAD%EMIS(JJ),ZF_R_WIN(JJ),B%XT_WIN1(JJ),    PTS_SR(JJ),T%XTAU_WR(JJ))
    PLW_S_TO_NR(JJ)  = ZLW(1.,                    T%TSNOW_ROAD%EMIS(JJ),T%XSVF_RS(JJ), ZT_S(JJ),         PTS_SR(JJ),T%XTAU_SR(JJ))

    PLW_NR_TO_WA(JJ) = ZLW(T%TSNOW_ROAD%EMIS(JJ),T%XEMIS_WALL(JJ),  ZF_W_NR(JJ),  PTS_SR(JJ),T%XT_WALL_A(JJ,1),T%XTAU_WR(JJ))    
    PLW_NR_TO_WB(JJ) = ZLW(T%TSNOW_ROAD%EMIS(JJ),T%XEMIS_WALL(JJ),  ZF_W_NR(JJ),  PTS_SR(JJ),T%XT_WALL_B(JJ,1),T%XTAU_WR(JJ))
    PLW_NR_TO_WIN(JJ)= ZLW(T%TSNOW_ROAD%EMIS(JJ),ZEMIS_WIN(JJ),         ZF_WIN_NR(JJ),PTS_SR(JJ),B%XT_WIN1(JJ),    T%XTAU_WR(JJ))
    PLW_NR_TO_S(JJ)  = ZLW(T%TSNOW_ROAD%EMIS(JJ),1.,                    ZF_S_NR(JJ),  PTS_SR(JJ),ZT_S(JJ),         T%XTAU_SR(JJ))
!
    IF (T%XURBTREE(JJ) .GT. 0.) THEN
      PLW_HV_TO_NR(JJ) = ZLW(PEMIS_HV(JJ),T%TSNOW_ROAD%EMIS(JJ),ZF_R_HV(JJ), PTS_HV(JJ), PTS_SR(JJ),1.)
      PLW_NR_TO_HV(JJ) = ZLW(T%TSNOW_ROAD%EMIS(JJ),PEMIS_HV(JJ),ZF_HV_R(JJ), PTS_SR(JJ),PTS_HV(JJ), 1.)
    ELSE
      PLW_HV_TO_NR(JJ) = 0.
      PLW_NR_TO_HV(JJ) = 0.
    ENDIF
!
  ENDIF
!
ENDDO
!-------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('URBAN_LW_COEF',1,ZHOOK_HANDLE)
!
END SUBROUTINE URBAN_LW_COEF

